#include <tamtypes.h>
#include <kernel.h>
#include "regs.h"
#include "gs.h"

// defines for SetGsCrt
#define NON_INTERLACED		0
#define INTERLACED		1
#define NTSC			2
#define PAL			3
#define FIELD			0
#define FRAME			1

int g_width = 640;
int g_height;

static int g_size;
static int g_width64;
static int g_currentFrame = 0;

int isPal(void)
{
	return *((char*)0x1FC7FF52) == 'E';
}

void initGS()
{
	g_height = isPal() ? 576 : 480;
	g_size = g_width * g_height / 2048;
	g_width64 = g_width / 64;
	
	// init video
	GS_CSR = 0x300;  // reset GS and GIF
	__asm__("sync.p");
	__asm__("nop");

	// set video mode. The kernel sets all the undocumented SMODE1 and SYNC bits
	SetGsCrt(INTERLACED, isPal() ? PAL : NTSC, FIELD);

	// mask interrupts
	GS_IMR = 0xff00;

	int magh = 4;

	GS_SET_PMODE(
		0,		// ReadCircuit1 OFF 
		1,		// ReadCircuit2 ON
		1,		// Use ALP register for Alpha Blending
		1,		// Alpha Value of ReadCircuit2 for output selection
		0,		// Blend Alpha with the output of ReadCircuit2
		0xFF		// Alpha Value = 1.0
	);

	GS_SET_DISPLAY2(
		656,		// X position in the display area (in VCK units)
		40,		// Y position in the display area (in Raster units)
		magh-1,		// Horizontal Magnification - 1
		0,		// Vertical Magnification = 1x
		g_width*magh-1,	// Display area width  - 1 (in VCK units) (Width*HMag-1)
		g_height-1	// Display area height - 1 (in pixels)	  (Height-1)
	);

	GS_SET_BGCOLOR(
		0,	// RED
		0,	// GREEN
		0	// BLUE
	);


	// No displacement between Primitive and Window coordinate systems.
	SetGSReg(GS_XYOFFSET_1,
		GS_XYOFFSET_DATA(
			0x0,
			0x0));

	// Clip to frame buffer.
	SetGSReg(GS_SCISSOR_1,
		GS_SCISSOR_DATA(
			0,
			g_width-1,
			0,
			g_height-1));
			
	// clear screens
	flip();
	SetGSReg(GS_RGBAQ, GS_SET_RGBAQ_DATA(0, 0, 0, 0, 0x3f800000));
	fillRect(0, 0, g_width, g_height);
	flip();
	fillRect(0, 0, g_width, g_height);
}

void waitVSync()
{
	GS_CSR = GS_CSR & 8;
	while(!(GS_CSR & 8));
}

void flip()
{
	int base0 = 0;
	int base1 = 0;
	if (g_currentFrame) {
		base0 = g_size;
	} else {
		base1 = g_size;
	}
	GS_SET_DISPFB2(
		base0,			// Frame Buffer base pointer = 0 (Address/2048)
		g_width64,		// Buffer Width (Pixels/64)
		PS2SS_GSPSMCT32,	// Pixel Storage Format
		0,			// Upper Left X in Buffer = 0
		0			// Upper Left Y in Buffer = 0
	);
	SetGSReg(GS_FRAME_1,
		GS_FRAME_DATA(
			base1,			// FrameBuffer base pointer = 0 (Address/2048)
			g_width64,		// Frame buffer width (Pixels/64)
			PS2SS_GSPSMCT32,	// Pixel Storage Format
			0));
	g_currentFrame ^= 1;
}

void fillRect(u16 x0, u16 y0, u16 x1, u16 y1)
{
	SetGSReg(GS_PRIM, PRIM_SPRITE);
	SetGSReg(GS_XYZ2, GS_SET_XYZ_DATA(x0<<4, y0<<4, 5));
	SetGSReg(GS_XYZ2, GS_SET_XYZ_DATA(x1<<4, y1<<4, 5));
}

void drawLine(int x0, int y0, int x1, int y1)
{
	SetGSReg(GS_PRIM, PRIM_LINE);
	SetGSReg(GS_XYZ2, GS_SET_XYZ_DATA(x0<<4, y0<<4, 5));
	SetGSReg(GS_XYZ2, GS_SET_XYZ_DATA(x1<<4, y1<<4, 5));
}

void setColor(int r, int g, int b) {
	SetGSReg(GS_RGBAQ, GS_SET_RGBAQ_DATA(r, g, b, 0, 0x3f800000));
}
